Package: betareg
Version: 3.2-5
Date: 2026-01-30
Title: Beta Regression
Authors@R: c(person(given = "Achim", family = "Zeileis", role = c("aut", "cre"), email = "Achim.Zeileis@R-project.org", comment = c(ORCID = "0000-0003-0918-3766")),
             person(given = "Francisco", family = "Cribari-Neto", role = "aut", email = "cribari@ufpe.br", comment = c(ORCID = "0000-0002-5909-6698")),
             person(given = "Bettina", family = "Grün", role = "aut", email = "Bettina.Gruen@wu.ac.at", comment = c(ORCID = "0000-0001-7265-4773")),
             person(given = "Ioannis", family = "Kosmidis", role = "aut", email = "ioannis.kosmidis@warwick.ac.uk", comment = c(ORCID = "0000-0003-1556-0302")),
             person(given = c("Alexandre", "B."), family = "Simas", role = "ctb", comment = "earlier version by"),
             person(given = c("Andrea", "V."), family = "Rocha", role = "ctb", comment = "earlier version by"))
Description: Beta regression for modeling beta-distributed dependent variables on the open unit interval (0, 1),
  e.g., rates and proportions, see Cribari-Neto and Zeileis (2010) <doi:10.18637/jss.v034.i02>.
  Moreover, extended-support beta regression models can accommodate dependent variables with
  boundary observations at 0 and/or 1, see Kosmidis and Zeileis (2025) <doi:10.1093/jrsssc/qlaf039>.
  For the classical beta regression model, alternative specifications are provided:
  Bias-corrected and bias-reduced estimation, finite mixture models, and recursive partitioning for
  beta regression, see Grün, Kosmidis, and Zeileis (2012) <doi:10.18637/jss.v048.i11>.
Depends: R (>= 3.6.0)
Imports: graphics, grDevices, methods, stats, flexmix, Formula, lmtest,
        modeltools, sandwich
Suggests: bamlss, car, distributions3 (>= 0.2.1), knitr, lattice,
        numDeriv, partykit, quarto, statmod, strucchange
License: GPL-2 | GPL-3
URL: https://topmodels.R-Forge.R-project.org/betareg/
BugReports: https://topmodels.R-Forge.R-project.org/betareg/contact.html
Encoding: UTF-8
VignetteBuilder: quarto
Author: Achim Zeileis [aut, cre] (<https://orcid.org/0000-0003-0918-3766>),
  Francisco Cribari-Neto [aut] (<https://orcid.org/0000-0002-5909-6698>),
  Bettina Grün [aut] (<https://orcid.org/0000-0001-7265-4773>),
  Ioannis Kosmidis [aut] (<https://orcid.org/0000-0003-1556-0302>),
  Alexandre B. Simas [ctb] (earlier version by),
  Andrea V. Rocha [ctb] (earlier version by)
Maintainer: Achim Zeileis <Achim.Zeileis@R-project.org>
Repository: R-Forge
Repository/R-Forge/Project: betareg
Repository/R-Forge/Revision: 365
Repository/R-Forge/DateTimeStamp: 2026-07-11 01:48:20
Date/Publication: 2026-07-11 01:48:20
NeedsCompilation: yes
Packaged: 2026-07-11 02:04:35 UTC; rforge
